import { ActionRow } from "components-jsx/ActionRow";
import { Button } from "components-jsx/Button";
import { ComponentMessage } from "components-jsx/ComponentMessage";
import { Container } from "components-jsx/Container";
import { Divider } from "components-jsx/Divider";
import { TextDisplay } from "components-jsx/TextDisplay";
import { ButtonStyles, User as OUser } from "oceanic.js";
import { Response } from "~/utils/types";
import { User } from "./User";
import { Constants } from "~/Constants";
import { ApplicationContent } from "./ApplicationContent";
import { StringSelect } from "components-jsx/StringSelect";
import { rejectReasons } from "~/utils/rejectReasons";

export function PendingApplicationMessage(props: {
	id: string;
	user: OUser;
	response: Response;
	locked?: boolean;
	threadID: string;
}) {
	const { id, user, response } = props;
	const locked = props.locked || false;
	return (
		<ComponentMessage>
			<TextDisplay>{"<@&1375617676139040909>"}</TextDisplay>
			<Container
				accentColor={
					!locked ? Constants.COLORS.NEW_REQ : Constants.COLORS.YELLOW
				}
			>
				<TextDisplay>## New join request</TextDisplay>
				<Divider />
				<User user={user} />
				<Divider />
				<ApplicationContent response={response} includeDetails={true} />
			</Container>
			<ActionRow>
				<Button
					style={ButtonStyles.SUCCESS}
					customID={`accept-${id}`}
					emoji={{
						name: "blobcatgreen",
						id: "1375806000312881233"
					}}
					disabled={locked}
				/>
				<Button
					style={ButtonStyles.SUCCESS}
					customID={`accept-friend-${id}`}
					emoji={{
						name: "hugcatcozy",
						id: "1375955808826687508"
					}}
					disabled={locked}
				/>
				<Button
					style={ButtonStyles.SECONDARY}
					customID={`interview-${id}`}
					emoji={{
						name: "💬"
					}}
				/>
			</ActionRow>
			<ActionRow>
				<Button
					style={ButtonStyles.DANGER}
					customID={`reject-${id}`}
					emoji={{
						name: "blobcatred",
						id: "1375806202470203513"
					}}
					disabled={locked}
				/>
				<Button
					style={ButtonStyles.DANGER}
					customID={`reject-ban-${id}`}
					emoji={{
						name: "BAN",
						id: "1375806319621046313"
					}}
					disabled={locked}
				/>
				<Button
					style={ButtonStyles.SECONDARY}
					customID={`${locked ? "un" : ""}lock-${id}`}
					emoji={{
						name: "🔑"
					}}
				/>
			</ActionRow>
			<ActionRow>
				<StringSelect
					customID="deny-reasons"
					placeholder="Deny reasons"
					disabled={locked}
					// @ts-expect-error
					options={Object.keys(rejectReasons).map((key) => {
						return {
							label: rejectReasons[key].shortDesc,
							description:
								rejectReasons[key].reason.length > 100
									? `${rejectReasons[key].reason.slice(
											0,
											97
									  )}...`
									: rejectReasons[key].reason,
							value: key
						};
					})}
				/>
			</ActionRow>
			<TextDisplay>-# Discuss in {`<#${props.threadID}>`}</TextDisplay>
		</ComponentMessage>
	);
}
