import { AllIntents, Client, ComponentTypes, MessageFlags } from "oceanic.js";
import { selfappReq } from "./utils/selfappReq";
import { setupJoinRequestHandler } from "./joinRequestHandler";
import { Constants } from "./Constants";
import { openDb } from "./utils/database";

export const client = new Client({
	auth: `Bot ${process.env.BOT_TOKEN}`,
	gateway: {
		intents: AllIntents,
		compress: false
	}
});

client.on("ready", async () => {
	console.log(`Logged in as ${client.user.tag}`);
	console.log("Checking user token");
	const selfappInfo: any = await selfappReq("/users/@me", "GET");
	if (!selfappInfo.username) console.error("Can't use selfapp");
	else console.log("Can use selfapp");

	setupJoinRequestHandler(client.shards.first()!);
});

client.on("messageCreate", (msg) => {
	if (msg.messageReference && msg.author.id === client.user.id) {
		if (!msg.messageReference.messageID) msg.delete();
	}
});

process.on("uncaughtException", (e) => {
	console.error(e);
});

export function start() {
	openDb();
	client.connect();
}

start();
